/*
	Phoenix Suite 2
	Author: Jean-Baptiste Wautier
	Copyright 2006-2009 (c)
*/
﻿namespace Phoenix.Log.Reader
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// Represents features to read the log of an IAppender
    /// </summary>
    public interface ILogReader
    {
        #region Properties

        /// <summary>
        /// Gets the log count.
        /// </summary>
        /// <returns>The number of log into the database</returns>        
        uint LogCount
        {
            get;
        }

        /// <summary>
        /// Gets the date of the newest log.
        /// </summary>
        /// <returns></returns>
        DateTime NewestLog
        {
            get;
        }

        /// <summary>
        /// Gets the date of the oldest log.
        /// </summary>
        /// <returns></returns>
        DateTime OldestLog
        {
            get;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Gets all the logs.
        /// </summary>
        /// <returns>The list of logs</returns>
        List<IRow> GetAllLogs();

        /// <summary>
        /// Gets the dependency of the log with the specified Guid.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <returns>
        /// The list of the dependency of the specified log
        /// </returns>
        List<IRow> GetChildren(Guid parent);

        /// <summary>
        /// Gets the dependency of the log with the specified Guid.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <returns>
        /// The list of the dependency of the specified log
        /// </returns>
        List<IRow> GetChildren(string parent);

        /// <summary>
        /// Gets the dependency of the specified log.
        /// </summary>
        /// <param name="baseLog">The base log.</param>
        /// <returns>The list of the dependency of the specified log</returns>
        List<IRow> GetChildren(IRow baseLog);

        /// <summary>
        /// Gets the row with the specified Guid.
        /// </summary>
        /// <param name="id">The GUID.</param>
        /// <returns>The row with the specified Guid</returns>
        IRow GetLogById(Guid id);

        /// <summary>
        /// Gets the row.
        /// </summary>
        /// <param name="id">The GUID.</param>
        /// <returns>The row with the specified Guid</returns>
        IRow GetLogById(string id);

        /// <summary>
        /// Gets the logs between.
        /// </summary>
        /// <param name="lowerThreshold">The lower threshold.</param>
        /// <param name="upperThreshold">The upper threshold.</param>
        /// <returns>Search result.</returns>
        List<IRow> GetLogsBetween(DateTime lowerThreshold, DateTime upperThreshold);

        /// <summary>
        /// Gets the logs between.
        /// </summary>
        /// <param name="lowerThreshold">The lower threshold. Should have that format: yyyy-MM-dd HH:mm:ss</param>
        /// <param name="upperThreshold">The upper threshold. Should have that format: yyyy-MM-dd HH:mm:ss</param>
        /// <returns>Search result.</returns>
        List<IRow> GetLogsBetween(string lowerThreshold, string upperThreshold);

        /// <summary>
        /// Gets all the logs of the specified type.
        /// </summary>
        /// <param name="stamp">Stamp of the log.</param>
        /// <returns>Search result</returns>
        List<IRow> GetLogsBy(Log.Stamp stamp);

        /// <summary>
        /// Gets all the logs of the specified list of types.
        /// </summary>
        /// <param name="logTypes">Stamp of the log.</param>
        /// <returns>Search result</returns>
        List<IRow> GetLogsBy(Log.Stamp[] logTypes);

        /// <summary>
        /// Gets the logs that contains the specified text into the exception message.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        List<IRow> GetLogsWithExceptionMessage(string value);

        /// <summary>
        /// Gets the logs that contains the specified text into the message.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        List<IRow> GetLogsWithMessage(string value);

        /// <summary>
        /// Gets the logs that contains the specified text into the title.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>Search result</returns>
        List<IRow> GetLogsWithTitle(string value);

        /// <summary>
        /// Gets the titles.
        /// </summary>
        /// <returns>A list of the different titles in the logs</returns>
        string[] GetTitles();

        /// <summary>
        /// Loads the statistics (Refresh the newest and oldest log and count the log entries).
        /// </summary>
        void LoadStatistics();

        #endregion Methods
    }
}