﻿namespace Phoenix.Tools.Translation
{
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Data;
    using System.Data.SQLite;

    /// <summary>
    /// Provides features to fetch the translation from the repository.
    /// In this case a SQLite database.
    /// </summary>
    public class SQLiteRepository : IRepository
    {
        #region Fields

        private string connectionString = string.Empty;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SQLiteRepository"/> class.
        /// </summary>
        protected SQLiteRepository()
        {
            ConnectionStringSettings connectionStringSettings = ConfigurationManager.ConnectionStrings["Translation"];
            if (connectionStringSettings == null)
                throw new ConfigurationErrorsException("Impossible to access the configuration file for the translator!");

            this.connectionString = connectionStringSettings.ConnectionString;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SQLiteRepository"/> class.
        /// </summary>
        /// <param name="dbpath">The path of the translation database.</param>
        protected SQLiteRepository(string dbpath)
        {
            this.connectionString = string.Format("Data Source={0};Version=3;", dbpath);
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Loads this instance.
        /// </summary>
        /// <exception cref="ConfigurationErrorsException"></exception>
        /// <returns>An instance of a SQLiteRepository</returns>
        public static SQLiteRepository Create()
        {
            return new SQLiteRepository();
        }

        /// <summary>
        /// Loads the specified path.
        /// </summary>
        /// <param name="path">The path.</param>
        /// <returns>An instance of a SQLiteRepository</returns>
        public static SQLiteRepository Create(string path)
        {
            return new SQLiteRepository(path);
        }

        /// <summary>
        /// Gets the count of translation stored in the repository. If any errors occured during
        /// the count, it returns -1. Note the information about the error is lost.
        /// </summary>
        /// <value>The count of translation.</value>
        public long CalculateCount()
        {
            string query = "SELECT COUNT(*) FROM BaseText;";
            return (long)this.QueryScalarDatabase(query);
        }

        /// <summary>
        /// Determines whether the specified key exist in the repository
        /// </summary>
        /// <param name="keyToCheck">The key to check.</param>
        /// <returns>
        /// 	<c>true</c> if the specified key exist in the repository. Otherwise <b>false</b>
        /// </returns>
        public bool ExistInRepository(string keyToCheck)
        {
            string query = string.Format(
                "SELECT COUNT(*) FROM BaseText WHERE value LIKE '{0}';", keyToCheck);
            object retObj = QueryScalarDatabase(query);

            if (retObj is long)
                return ((long)retObj > 0);
            return false;
        }

        /// <summary>
        /// Gets the translations from the translation repository.
        /// </summary>
        /// <returns>
        /// A dictionary list of the text and its translation.
        /// </returns>
        public Dictionary<string, string> GetTranslations()
        {
            using (DataSet dataSet = QueryDatabase("SELECT * FROM Translations;"))
            {

                Dictionary<string, string> dictionary = new Dictionary<string, string>();

                if (dataSet.Tables.Count == 0)
                    return dictionary;

                foreach (DataRow dataRow in dataSet.Tables[0].Rows)
                {
                    string translation = (dataRow["TranslatedValue"].Equals(DBNull.Value))
                        ? (string)dataRow["Value"]
                        : (string)dataRow["TranslatedValue"];
                    dictionary.Add(((string)dataRow["Value"]).ToLower(), translation);
                }

                return dictionary;
            }
        }

        /// <summary>
        /// Inserts the new phrase into the database.
        /// By default the database assumes this phrase is in english
        /// and stores that into the english language.
        /// </summary>
        /// <param name="phrase">The phrase.</param>
        public void InsertNewPhrase(string phrase)
        {
            if (string.IsNullOrEmpty(phrase))
                throw new ArgumentException("Please insert a not null or empty phrase in the dictionary!", "phrase");
            string query = string.Format("INSERT INTO BaseText (Value) VALUES ('{0}');"
                , phrase.Replace("\'", "\'\'"));
            NonQueryDatabase(query);
        }

        /// <summary>
        /// Sets the languageToSet for the translation.
        /// Change the languageToSet does NOT refresh the translation. To be up to date, the
        /// call of GetTranslation is mandatory!
        /// </summary>
        /// <param name="languageToSet">The translation will be on this specified languageToSet.</param>
        public void SetLanguage(Language language)
        {
            string query = string.Format(
                "UPDATE Language SET isActive = 1 WHERE Abreviation = '{0}';"
                , language.ToString());
            NonQueryDatabase(query);
        }

        private int NonQueryDatabase(string query)
        {
            int ret;
            using (SQLiteConnection connection = new SQLiteConnection(this.connectionString))
            using (SQLiteCommand command = new SQLiteCommand(connection))
            {
                command.CommandType = CommandType.Text;
                command.CommandText = query;

                connection.Open();
                ret = (int)(long)command.ExecuteNonQuery();
                connection.Close();
            }
            return ret;
        }

        private DataSet QueryDatabase(string query)
        {
            DataSet dataSet = new DataSet();
            using (SQLiteConnection connection = new SQLiteConnection(this.connectionString))
            using (SQLiteCommand command = new SQLiteCommand(connection))
            {
                command.CommandType = CommandType.Text;
                command.CommandText = query;

                SQLiteDataAdapter dataAdapter = new SQLiteDataAdapter(command);
                dataAdapter.Fill(dataSet);
            }
            return dataSet;
        }

        private object QueryScalarDatabase(string query)
        {
            using (SQLiteConnection connection = new SQLiteConnection(this.connectionString))
            using (SQLiteCommand command = new SQLiteCommand(connection))
            {
                command.CommandType = CommandType.Text;
                command.CommandText = query;

                connection.Open();
                object retObject = command.ExecuteScalar();
                connection.Close();
                return retObject;
            }
        }

        #endregion Methods
    }
}